<?php
/*
Plugin Name: Bricks Mega Menu Extension
Description: This plugin extends the Bricks Element "Nav Menu" with a Mega Menu functionality.
Version: 1.5
Author: WP Monkey
Author URI: www.wp-monkey.com
License: GPL2
License URI: https://www.gnu.org/licenses/gpl-2.0.html
*/

if ( !defined( 'ABSPATH' ) ) exit;

require "includes/admin-menu-walker.php";

class Wpm_Bricks_Mega_Menu {

  function __construct() {
    $this->init();
  }

  function init() {
    $this->register_constants();
    $this->customize_admin_menu();
    $this->customize_bricks_builder();
    add_filter( 'plugin_row_meta', array($this, 'additional_plugin_links'), 10, 2 );
    add_action('wp_enqueue_scripts', array($this, 'load_assets'), 10);
    add_action('wp_ajax_wpm_megamenu_get_settings', array($this, 'megamenu_get_settings'), 10);
    add_action( 'wp_ajax_nopriv_wpm_megamenu_get_settings', array($this, 'megamenu_get_settings'), 10);
  }

  private function register_constants() {
    define('WPM_PLUGIN_DIR', plugins_url('', dirname(__FILE__) ) . '/bricks-mega-menu');
  }

  public function load_assets() {
    wp_enqueue_style( 'wpm-mega-menu', WPM_PLUGIN_DIR . '/assets/css/megamenu.css' );
    wp_enqueue_script( 'wpm-mega-menu', WPM_PLUGIN_DIR . '/assets/js/megamenu.js', array('jquery'), true );

    $output_array = array();

    if (get_option('wpm_megamenu_settings')) {
      $options = get_option('wpm_megamenu_settings', true);
      $menuID = $options['megaMenuNavMenu'];

      $navigation = wp_get_nav_menu_items($menuID);

      if ($navigation) {
        foreach ( $navigation as $navItem ) {

          if ($navItem->wpm_megamenu && $navItem->wpm_megamenu != "none") {
            $output = "<div data-nav-item='menu-item-". $navItem->ID . "' class='wpm-mega-menu'>";
            $output .= do_shortcode($navItem->wpm_megamenu);
            $output .= "</div>";
    
            array_push($output_array, $output);
          }
        }
      } 
    }

    $params = array(
      'data' => $output_array,
      'ajaxurl' => admin_url('admin-ajax.php')
    );

    wp_localize_script( 'wpm-mega-menu', 'MegaMenuSettings', $params );
  }

  private function customize_admin_menu() {
    add_filter( 'wp_edit_nav_menu_walker', 'wpm_custom_nav_edit_walker',10,2 );
    function wpm_custom_nav_edit_walker($walker,$menu_id) {
        return 'Wpm_Walker_Nav_Menu_Edit_Custom';
    }

    // Saves new field navmenu
    add_action('wp_update_nav_menu_item', 'wpm_custom_nav_update',10, 3);
    function wpm_custom_nav_update($menu_id, $menu_item_db_id, $args ) {
        if ( is_array($_REQUEST['menu-item-bricks-template']) ) {
            $custom_value = $_REQUEST['menu-item-bricks-template'][$menu_item_db_id];
            update_post_meta( $menu_item_db_id, '_menu_item_bricks_template', $custom_value );
        }
    }

    // Adds value of new field to Navmenu
    add_filter( 'wp_setup_nav_menu_item','wpm_bricks_template_nav_item' );
    function wpm_bricks_template_nav_item($menu_item) {
        $menu_item->wpm_megamenu = get_post_meta( $menu_item->ID, '_menu_item_bricks_template', true );
        return $menu_item;
    }
  }

  public function additional_plugin_links( $links, $file ) {    
    if ( plugin_basename( __FILE__ ) == $file ) {
        $row_meta = array(
          'docs'    => '<a href="' . esc_url( 'https://www.wp-monkey.com/bricks-mega-menu-extension' ) . '" target="_blank" aria-label="' . esc_attr__( 'How To Use', 'domain' ) . '" style="color:purple;">' . esc_html__( 'How To Use', 'domain' ) . '</a>'
        );
        return array_merge( $links, $row_meta );
    }
    return (array) $links;
  }

  public function megamenu_get_settings() {
    if (get_option("wpm_megamenu_settings")) {
      echo json_encode(get_option("wpm_megamenu_settings", true));
    }
    
    die;
  }

  private function customize_bricks_builder() {
    add_filter( 'bricks/elements/nav-menu/control_groups', function( $control_groups ) {
      $control_groups['wpmMegaMenu'] = [
          'tab'      => 'content', 
          'title'    => esc_html__( 'Mega Menu', 'textdomain' ),
      ];
    
      return $control_groups;
    } );
    
    /**
     * Add Controls for the new group
     */
    
    add_filter( 'bricks/elements/nav-menu/controls', function( $controls ) {
      $controls['activateMegaMenu'] = [
          'tab'      => 'content',
          'group'    => 'wpmMegaMenu',
          'label'    => esc_html__( 'Activate Mega Menu', 'textdomain' ),
          'type'     => 'checkbox'
      ];
    
      return $controls;
    } );
    
    add_filter( 'bricks/elements/nav-menu/controls', function( $controls ) {
      $controls['activateMegaMenu'] = [
          'tab'      => 'content',
          'group'    => 'wpmMegaMenu',
          'label'    => esc_html__( 'Activate Mega Menu', 'bricks' ),
          'type'     => 'checkbox',
          'default' => true
      ];

      $controls['megaMenuTrigger'] = [
        'tab' => 'content',
        'group'    => 'wpmMegaMenu',
        'label' => esc_html__( 'Trigger', 'bricks' ),
        'type' => 'select',
        'options' => [
          'click' => esc_html__( 'Click', 'bricks' ),
          'hover' => esc_html__( 'Hover', 'bricks' ),
        ],
        'inline' => true,
        'placeholder' => esc_html__( 'Trigger', 'bricks' ),
        'default' => 'hover',
      ];

      $controls['megaMenuFullWidth'] = [
        'tab'      => 'content',
        'group'    => 'wpmMegaMenu',
        'label'    => esc_html__( 'Full Width', 'bricks' ),
        'type'     => 'checkbox',
        'default'  => true
      ];

      $controls['megaMenuTopSpacing'] = [
        'tab' => 'content',
        'group'    => 'wpmMegaMenu',
        'label' => esc_html__( 'Top Spacing', 'bricks' ),
        'type' => 'number',
        'min' => 0,
        'step' => 1, 
        'unit' => 'px',
        'inline' => true,
        'default' => 0,
      ];

      $controls['megaMenuInfo'] = [
        'tab' => 'content',
        'group'    => 'wpmMegaMenu',
        'content' => esc_html__( 'Currently, a workaround is needed to update the settings for the Mega Menu. Once you have pressed "Save", update your browser in the builder. After that the settings will be applied.', 'bricks' ),
        'type' => 'info',
      ];
    
      return $controls;
    } );

    add_filter( 'bricks/element/settings', function( $settings, $element ) {

      if ($element->name == "nav-menu" && bricks_is_builder() === true) {
        $megamenu_settings = array(
          "activateMegaMenu" => isset($settings['activateMegaMenu']) ? $settings['activateMegaMenu'] : false,
          "megaMenuTrigger" => isset($settings['megaMenuTrigger']) ? $settings['megaMenuTrigger'] : "hover",
          "megaMenuFullWidth" => isset($settings['megaMenuFullWidth']) ? $settings['megaMenuFullWidth'] : false,
          "megaMenuTopSpacing" => isset($settings['megaMenuTopSpacing']) ? $settings['megaMenuTopSpacing'] : 0,
          "megaMenuNavMenu" => isset($settings['menu']) ? $settings['menu'] : "",
        );
        update_option("wpm_megamenu_settings", $megamenu_settings);
      }

      return $settings;
    }, 10, 2 );

  }
}
new Wpm_Bricks_Mega_Menu();